@@ -4,13 +4,14 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
+from io import StringIO
 from .strings import escape
 
 
 class PoFile(object):
     def __init__(self):
         self.header_fields = []
-        self.header_index = {}
+        self._header_index = {}
         self.entries = {}
 
     def clone(self):
@@ -21,10 +22,10 @@ class PoFile(object):
         return po_file
 
     def add_header_field(self, field, value):
-        if field in self.header_index:
-            self.header_fields[self.header_index] = (field, value)
+        if field in self._header_index:
+            self.header_fields[self._header_index[field]] = (field, value)
         else:
-            self.header_index[field] = len(self.header_fields)
+            self._header_index[field] = len(self.header_fields)
             self.header_fields.append((field, value))
 
     def add_entry(self, message, plural=None, context=None):
@@ -48,22 +49,29 @@ class PoFile(object):
             for field, value in self.header_fields:
                 print(r'"{}: {}\n"'.format(field, value), file=fp)
             needs_blank_line = True
-        nplurals = self._get_nplurals()
-        for entry in sorted(self.entries.values(), key=lambda e: e.locations):
+        nplurals = self.get_nplurals()
+        for entry in sorted(self.entries.values(), key=get_entry_sort_key):
             if needs_blank_line:
                 print('', file=fp)
             needs_blank_line = entry.dump(
                 fp, nplurals, include_locations=include_locations, prune_obsolete=prune_obsoletes)
 
+    def dumps(self, include_locations=True, prune_obsoletes=False):
+        string_file = StringIO()
+        self.dump(string_file, include_locations, prune_obsoletes)
+        return string_file.getvalue()
+
     def get_catalog(self):
         catalog = {}
         for entry in self.entries.values():
             entry.fill_catalog(catalog)
         return catalog
 
-    def _get_nplurals(self):
-        for field, value in self.header_fields:
-            if field == 'Plural-forms':
+    def get_nplurals(self):
+        plural_field_index = self._header_index.get('Plural-Forms', -1)
+        if plural_field_index != -1:
+            field, value = self.header_fields[plural_field_index]
+            if field == 'Plural-Forms':
                 for pair in value.split(';'):
                     parts = pair.partition('=')
                     if parts[0].strip() == 'nplurals':
@@ -139,3 +147,6 @@ def get_msgid(message, context=None):
         return '{}\x04{}'.format(context, message)
     else:
         return message
+
+def get_entry_sort_key(entry):
+    return entry.locations, entry.context if entry.context else '', entry.message
